@@ -22,7 +22,7 @@ our %EXPORT_TAGS = ( 'all' => \@EXPORT_OK, );
 sub _CERTIFICATE_HEADER_SIZE { return 40; }
 sub _MAX_PUBLIC_KEY_SIZE     { return 65_536; }
 
-our $VERSION = '0.11';
+our $VERSION = '0.12';
 
 sub pin_sha256 {
     my ($path) = @_;
@@ -38,10 +38,18 @@ sub pin_sha256 {
         $pem_encoded_public_key_string =
           _process_pem_x509_certificate( $handle, $file_header, $path );
     }
-    elsif ( $file_header =~ /^[-]{5}BEGIN[ ](?:RSA[ ])?PUBLIC[ ]KEY[-]{5}/smx )
+    elsif ( $file_header =~
+        /^[-]{5}BEGIN[ ](?:RSA[ ])?(PUBLIC|PRIVATE)[ ]KEY[-]{5}/smx )
     {
-        $pem_encoded_public_key_string =
-          _process_pem_public_key( $handle, $file_header, $path );
+        my ($type) = ($1);
+        if ( $type eq 'PRIVATE' ) {
+            $pem_encoded_public_key_string =
+              _process_pem_private_key( $handle, $file_header, $path );
+        }
+        else {
+            $pem_encoded_public_key_string =
+              _process_pem_public_key( $handle, $file_header, $path );
+        }
     }
     elsif ( $file_header =~
         /^[-]{5}BEGIN[ ](?:NEW[ ])?CERTIFICATE[ ]REQUEST[-]{5}/smx )
@@ -54,6 +62,7 @@ sub pin_sha256 {
         $pem_encoded_public_key_string =
           _check_for_der_encoded_x509_certificate( $handle, $file_header,
             $path )
+          || _check_for_der_encoded_private_key( $handle, $file_header, $path )
           || _check_for_der_encoded_public_key( $handle, $file_header, $path );
         if ( !defined $pem_encoded_public_key_string ) {
             Carp::croak("$path is not an X.509 Certificate");
@@ -150,6 +159,40 @@ __SUBJECT_PUBLIC_KEY_INFO__
     return $pem_encoded_public_key_string;
 }
 
+sub _check_for_der_encoded_private_key {
+    my ( $handle, $file_header, $path ) = @_;
+    my $pem_encoded_public_key_string;
+    seek $handle, 0, Fcntl::SEEK_SET()
+      or Carp::croak("Failed to seek to start of $path:$EXTENDED_OS_ERROR");
+    defined read $handle, my $der_encoded_private_key_string,
+      _MAX_PUBLIC_KEY_SIZE()
+      or Carp::croak("Failed to read from $path:$EXTENDED_OS_ERROR");
+    my $pem_encoded_private_key_string =
+        "-----BEGIN RSA PRIVATE KEY-----\n"
+      . MIME::Base64::encode_base64($der_encoded_private_key_string)
+      . "-----END RSA PRIVATE KEY-----\n";
+    eval {
+        my $privkey =
+          Crypt::OpenSSL::RSA->new_private_key($pem_encoded_private_key_string);
+        $pem_encoded_public_key_string = $privkey->get_public_key_x509_string();
+    } or do {
+        return;
+    };
+    return $pem_encoded_public_key_string;
+}
+
+sub _process_pem_private_key {
+    my ( $handle, $file_header, $path ) = @_;
+    my $pem_encoded_public_key_string;
+    seek $handle, 0, Fcntl::SEEK_SET()
+      or Carp::croak("Failed to seek to start of $path:$EXTENDED_OS_ERROR");
+    defined read $handle, my $rsa_private_key_string, _MAX_PUBLIC_KEY_SIZE()
+      or Carp::croak("Failed to read from $path:$EXTENDED_OS_ERROR");
+    my $privkey = Crypt::OpenSSL::RSA->new_private_key($rsa_private_key_string);
+    $pem_encoded_public_key_string = $privkey->get_public_key_x509_string();
+    return $pem_encoded_public_key_string;
+}
+
 sub _process_pem_public_key {
     my ( $handle, $file_header, $path ) = @_;
     my $pem_encoded_public_key_string;
@@ -222,7 +265,7 @@ This module allows the calculation of RFC 7469 HTTP Public Key Pin header values
 
 =head2 pin_sha256
 
-This function accepts the path to a L<X.509 Certificate|http://tools.ietf.org/html/rfc5280>.  It will load the public key from the certificate and prepare the appropriate value for the pin_sha256 parameter of the Public-Key-Pins value. This function will also make an attempt to read public keys (in PEM (L<SubjectPublicKeyInfo|http://tools.ietf.org/html/rfc5280#section-4.1.2.7> or L<PKCS#1|https://tools.ietf.org/html/rfc2437>) or DER format) and L<PKCS#10 Certificate Requests|https://tools.ietf.org/html/rfc2986> in PEM or DER format.
+This function accepts the path to a L<X.509 Certificate|http://tools.ietf.org/html/rfc5280>.  It will load the public key from the certificate and prepare the appropriate value for the pin_sha256 parameter of the Public-Key-Pins value. This function will also make an attempt to read public keys (in PEM (L<SubjectPublicKeyInfo|http://tools.ietf.org/html/rfc5280#section-4.1.2.7> or L<PKCS#1|https://tools.ietf.org/html/rfc2437>) or DER format), private keys (in PEM L<PKCS#1|https://tools.ietf.org/html/rfc2437> or DER format) and L<PKCS#10 Certificate Requests|https://tools.ietf.org/html/rfc2986> in PEM or DER format.
 
 =head1 SUBROUTINES/METHODS
 
@@ -234,15 +277,15 @@ None.  This module only has the one exported function.
  
 =item C<< Failed to open %s for reading >>
  
-Failed to open the supplied X.509 Certificate, PKCS10 Certificate Request or Public Key file
+Failed to open the supplied X.509 Certificate, PKCS10 Certificate Request, Private or Public Key file
  
 =item C<< Failed to read from %s >>
  
-Failed to read from the X.509 Certificate, PKCS10 Certificate Request or Public Key file
+Failed to read from the X.509 Certificate, PKCS10 Certificate Request, Private or Public Key file
 
-=item C<< %s is not an X.509 Certificate, PKCS10 Certificate Request or Public Key >>
+=item C<< %s is not an X.509 Certificate, PKCS10 Certificate Request, Private or Public Key >>
  
-The supplied input file does not look like X.509 Certificate File, PKCS10 Certificate Request or Public Key. These files may be encoded in PEM or DER format. A PEM encoded X.509 Certificate file has the following header
+The supplied input file does not look like X.509 Certificate File, PKCS10 Certificate Request, Private or Public Key. These files may be encoded in PEM or DER format. A PEM encoded X.509 Certificate file has the following header
 
   -----BEGIN CERTIFICATE-----
 
@@ -254,6 +297,10 @@ A PEM encoded PKCS#1 Public Key has the following header
 
   -----BEGIN RSA PUBLIC KEY-----
 
+A PEM encoded PKCS#1 Private Key has the following header
+
+  -----BEGIN RSA PRIVATE KEY-----
+
 A PEM encoded SubjectPublicKeyInfo Public Key has the following header
 
   -----BEGIN PUBLIC KEY-----
